﻿using ff14bot;
using ff14bot.Enums;
using ff14bot.Managers;
using ff14bot.Objects;
using TreeSharp;

namespace Heathcliff
{
    internal class DarkKnightRoutine : CombatRoutine
    {
        private const uint HARD_SLASH = 3617;
        private const uint SPINNING_SLASH = 3619;
        private const uint SCOURGE = 3620;
        private const uint SYPHON_STRIKE = 3623;
        private const int DARKSIDE_AURA = 751;
        private const int SCOURGE_AURA = 741;
        private const uint BLOOD_WEAPON = 3625;
        private const uint REPRISAL = 3626;
        private const uint POWER_SLASH = 3627;
        private const uint CONVALESCENSE = 12;
        private const uint BLOOD_PRICE = 3631;
        private const uint SOULEATER = 3632;

        public override ClassJobType Job
        {
            get { return ClassJobType.Paladin; }
        }

        protected override Composite CreateBehavior()
        {
            var hardSlash = Spells.CastOnTarget(HARD_SLASH);
            var souleater = new Decorator(c => UseSouleater, Spells.CastOnTarget(SOULEATER));
            var syphonStrike = new Decorator(c => UseSyphonStrike, Spells.CastOnTarget(SYPHON_STRIKE));
            var spinningSlash = new Decorator(c => UseSpinningSlash, Spells.CastOnTarget(SPINNING_SLASH));
            var powerSlash = new Decorator(c => UsePowerSlash, Spells.CastOnTarget(POWER_SLASH));
            var convalescense = new Decorator(c => UseCovalescense, Spells.CastOnSelf(CONVALESCENSE));
            var scourge = new Decorator(c => UseScourge, Spells.CastOnTarget(SCOURGE));
            var bloodweapon = Spells.CastOnSelf(BLOOD_WEAPON);
            var reprisal = Spells.CastOnTarget(REPRISAL);
            var bloodPrice = new Decorator(c => UseBloodPrice, Spells.CastOnSelf(BLOOD_PRICE));

            var rotation = new PrioritySelector(bloodweapon, convalescense, reprisal, bloodPrice, powerSlash,
                souleater, syphonStrike, spinningSlash, scourge, hardSlash);

            return new Decorator(c => InCombat, rotation);
        }

        private static bool InCombat
        {
            get { return Core.Me.InCombat && Core.Me.CurrentTarget is BattleCharacter; }
        }

        private static bool UseSouleater
        {
            get { return Actionmanager.LastSpellId == SYPHON_STRIKE; }
        }

        private static bool UseBloodPrice
        {
            get { return Core.Me.CurrentManaPercent < 100; }
        }

        private static bool UseScourge
        {
            get
            {
                var target = Core.Me.CurrentTarget as BattleCharacter;
                return target != null && !target.HasMyAura(SCOURGE_AURA);
            }
        }

        private static bool UseCovalescense
        {
            get { return Core.Me.CurrentHealthPercent < 50; }
        }

        private static bool UseSyphonStrike
        {
            get 
            { 
                return (Core.Me.CurrentManaPercent < 60 || UseSouleaterCombo) 
                    && Actionmanager.LastSpellId == HARD_SLASH; 
            }
        }

        private static bool UseSouleaterCombo
        {
            get { return Actionmanager.HasSpell(SOULEATER) && Core.Me.CurrentHealthPercent < 60; }
        }

        private static bool UseSpinningSlash
        {
            get { return Actionmanager.LastSpellId == HARD_SLASH; }
        }

        private static bool UsePowerSlash
        {
            get { return Actionmanager.LastSpellId == SPINNING_SLASH; }
        }
    }
}
