﻿using ff14bot;
using ff14bot.Enums;
using ff14bot.Managers;
using ff14bot.Objects;
using TreeSharp;

namespace Heathcliff
{
    internal class PaladinRoutine : CombatRoutine
    {
        private const uint FAST_BLADE = 9;
        private const uint SAVAGE_BLADE = 11;
        private const uint RIOT_BLADE = 15;
        private const uint RAGE_OF_HALONE = 21;
        private const uint AWARENESS = 13;
        private const uint TEMPERED_WILL = 19;
        private const uint BLOODBATH = 34;
        private const uint SPIRITS_WITHIN = 29;
        private const uint SHIELD_SWIPE = 25;

        public override ClassJobType Job
        {
            get { return ClassJobType.Paladin; }
        }

        protected override Composite CreateBehavior()
        {
            var fastBlade = Spells.CastOnTarget(FAST_BLADE);
            var riotBlade = new Decorator(c => UseRiot, Spells.CastOnTarget(RIOT_BLADE));
            var savageBlade = new Decorator(c => UseSavage, Spells.CastOnTarget(SAVAGE_BLADE));
            var rageOfHalone = new Decorator(c => UseRage, Spells.CastOnTarget(RAGE_OF_HALONE));
            var awareness = new Decorator(c => UseAwareness, Spells.CastOnTarget(AWARENESS));
            var tempered = new Decorator(c => UseTempered, Spells.CastOnTarget(TEMPERED_WILL));
            var bloodbath = new Decorator(c => UseBloodbath, Spells.CastOnTarget(BLOODBATH));
            var spirits = Spells.CastOnTarget(SPIRITS_WITHIN);
            var swipe = Spells.CastOnTarget(SHIELD_SWIPE);

            var rotation = new PrioritySelector(bloodbath, tempered, awareness, spirits, riotBlade, savageBlade,
                rageOfHalone, swipe, fastBlade);

            return new Decorator(c => InCombat, rotation);
        }

        private static bool InCombat
        {
            get { return Core.Me.InCombat && Core.Me.CurrentTarget is BattleCharacter; }
        }

        private static bool UseBloodbath
        {
            get { return Core.Me.CurrentHealthPercent < 60; }
        }

        private static bool UseRiot
        {
            get { return Core.Me.CurrentManaPercent < 35 && Actionmanager.LastSpellId == FAST_BLADE; }
        }

        private static bool UseTempered
        {
            get
            {
                return Core.Me.HasAura(280)
                       || Core.Me.HasAura(564)
                       || Core.Me.HasAura(13)
                       || Core.Me.HasAura(457)
                       || Core.Me.HasAura(240)
                       || Core.Me.HasAura(630)
                       || Core.Me.HasAura(14);
            }
        }

        private static bool UseSavage
        {
            get { return Actionmanager.LastSpellId == FAST_BLADE; }
        }

        private static bool UseRage
        {
            get { return Actionmanager.LastSpellId == SAVAGE_BLADE; }
        }

        private static bool UseAwareness
        {
            get { return Core.Me.CurrentHealthPercent <= 70; }
        }
    }
}
